// vim: set undofile:
// Program: multi_noncopyable

// Ako zelimo da napravimo klasu za koju zelimo da
// zabranimo kopiranje i operator dodele, dovoljno
// je da ih definisemo kao privatne, i da ih ne
// implementiramo.
// Ako zelimo da napravimo vise takvih klasa,
// ovo ponasanje mozemo da ubacimo u neku od
// nadklasa.

#include <string>
#include <iostream>

// Kompletnu implementaciju ove klase mozete naci u
// boost biblioteci - boost::noncopyable
class non_copyable {
protected:
    // ne zelimo da iko moze da napravi direktno instancu
    // nase klase - moze da pozove konstruktor ili destruktor
    // samo ako nasledi non_copyable
    non_copyable() {}
    ~non_copyable() {}

private:
    // deklarisemo konstruktor kopije i operator dodele
    // kao privatne, i ne implementiramo ih
    non_copyable(const non_copyable & other);
    const non_copyable& operator = (const non_copyable & other);
};


// Pravimo klasu koja 'barata' bazom podataka
class database {
public:
    database(const std::string & url)
        : m_url(url)
    {
        std::cout << "Povezujem se sa bazom: " << url << std::endl;
    }

private:
    std::string m_url;
};


// Pravimo klasu koja barata internom bazom.
// Implementacija se oslanja na to da mozemo imati
// samo jednu konekciju ka internoj bazi
class internal_database: public database, public non_copyable {
public:
    internal_database()
        : database("http://localhost/")
    {
    }

};


int main(int argc, const char *argv[])
{
    database db1("http://sql.matf.bg.ac.rs/");

    // Kopiranje baze radi bez problema
    database db1_copy(db1);

    internal_database db2;

    // Ovo ne prolazi - kompajler javlja poruku o gresci
    internal_database db2_copy(db2);

    return 0;
}
