// vim: set undofile:
// Program: student

#include <string>
#include <vector>
#include <algorithm>
#include <iostream>

class student {
public:
    student(const std::string & name)
        : m_name(name)
    {
    }

    double average() const
    {
        return std::accumulate(m_grades.cbegin(), m_grades.cend(), 0) / (double)m_grades.size();
    }

    void add_grade(int grade)
    {
        m_grades.push_back(grade);
    }

    virtual std::string name() const
    {
        return m_name;
    }

private:
    std::string m_name;
    std::vector<int> m_grades;

};


class master_student: public student {
public:
    master_student(const std::string & name, const std::string & thesis, const std::string & mentor)
        : student(name), m_thesis(thesis), m_mentor(mentor)
    {
    }

    virtual std::string name() const
    {
        return student::name() + " MSC";
    }

private:
    std::string m_thesis;
    std::string m_mentor;
};


int main(int argc, const char *argv[])
{
    student s1("Petar Petrovic");

    s1.add_grade(8);
    s1.add_grade(9);
    s1.add_grade(8);
    s1.add_grade(9);

    master_student s2("Janka Jankovic", "Plan 9 FS", "Mirko Mirkovic");

    s2.add_grade(9);
    s2.add_grade(10);
    s2.add_grade(9);

    // Student s3 je isto sto i s2, samo menjamo klasu
    student & s3(s2);


    std::cout << s1.name() << " " << s1.average() << std::endl;
    std::cout << s2.name() << " " << s2.average() << std::endl;

    // Sta se dobija ako ispisujemo za s3?
    // Gubi se podatak da je s3 master student, ili se samo poziva metod name
    // iz studenta, a ne iz master_student klase?
    // Cak i ako stavimo da je name virtuelan metod, ne radi ono sto ocekujemo.
    // Deklaracija 'student s3(s2)' kaze da zelimo da nam tip bude tacno student,
    // a ne nista drugo. Ova pojava se zove 'slicing'. Zato izuzetke hvatamo kao
    // kontantne reference, a ne kao vrednosti.
    // Ako stavimo da je s3 pokazivac ili referenca na s2, onda radi.
    std::cout << s3.name() << " " << s3.average() << std::endl;

    return 0;
}
